/*
 * Copyright (C) 2005 Andre Noll <maan@systemlinux.org>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */


#include <openssl/pem.h>
#include "para.h"

#define LOAD_PUBLIC_KEY 0
#define LOAD_PRIVATE_KEY 1

static EVP_PKEY *load_key(const char *file, int private)
{
	BIO *key;
	EVP_PKEY *pkey = NULL;

	key = BIO_new(BIO_s_file());
	if (!key)
		return NULL;
	if (BIO_read_filename(key, file) > 0)
		pkey = private?
			PEM_read_bio_PrivateKey(key, NULL, NULL, NULL) :
			PEM_read_bio_PUBKEY(key, NULL, NULL, NULL);
	BIO_free(key);
	return pkey;
}

static int get_key(char *key_file, RSA **rsa, int private)
{
	EVP_PKEY *pkey;

	pkey = load_key(key_file, private);
	if (!pkey)
		return -1; /* can not read private key */
	*rsa = EVP_PKEY_get1_RSA(pkey);
        EVP_PKEY_free(pkey);
        if (!*rsa)
		return -2; /* rsa error */
	return RSA_size(*rsa);
}

int para_decrypt_buffer(char *key_file, unsigned char *outbuf, unsigned char *inbuf, int rsa_inlen)
{
	RSA *rsa;
	int ret, rsa_outlen, keysize = get_key(key_file, &rsa, LOAD_PRIVATE_KEY);

	if (keysize < 0)
		return -1;
	rsa_outlen = RSA_private_decrypt(rsa_inlen, inbuf, outbuf, rsa, RSA_PKCS1_PADDING);
	ret = -2; /* decrypt error */
	if (rsa_outlen > 0)
		ret = 1;
	return ret;
}

int para_decrypt(char *key_file, long unsigned *challenge_nr, unsigned char *buf, int rsa_inlen)
{
	unsigned char *rsa_out = OPENSSL_malloc(128);
	int ret = para_decrypt_buffer(key_file, rsa_out, buf, rsa_inlen);

	if (ret > 0)
		ret = sscanf((char *)rsa_out, "%lu", challenge_nr) == 1? 1: -42;
	OPENSSL_free(rsa_out);
	return ret;
}

int para_encrypt_buffer(char *key_file, unsigned char *inbuf,
		const unsigned len, unsigned char *outbuf)
{
	RSA *rsa;
	int ret = get_key(key_file, &rsa, LOAD_PUBLIC_KEY);

	if (ret >= 0)
		ret = RSA_public_encrypt(len, inbuf, outbuf, rsa,
			RSA_PKCS1_PADDING);
	return ret;
}

int para_encrypt(char *key_file, long unsigned challenge_nr, unsigned char *outbuf)
{
	unsigned char *inbuf = (unsigned char*) make_message("%lu", challenge_nr);
	int ret = para_encrypt_buffer(key_file, inbuf, strlen(inbuf), outbuf);
	free(inbuf);
	return ret;
}

